@@ -4,6 +4,7 @@ use Dancer ':syntax';
 use Dancer::Plugin::Ajax;
 
 use Dancer::Plugin::DBIC;
+use Dancer::Plugin::Auth::Extensible;
 
 use Socket6 (); # to ensure dependency is met
 use HTML::Entities (); # to ensure dependency is met
@@ -18,6 +19,7 @@ use App::Netdisco::Web::Report;
 use App::Netdisco::Web::AdminTask;
 use App::Netdisco::Web::TypeAhead;
 use App::Netdisco::Web::PortControl;
+use App::Netdisco::Web::About;
 
 sub _load_web_plugins {
   my $plugin_list = shift;
@@ -43,6 +45,10 @@ if (setting('extra_web_plugins') and ref [] eq ref setting('extra_web_plugins'))
     _load_web_plugins( setting('extra_web_plugins') );
 }
 
+# workaround for https://github.com/PerlDancer/Dancer/issues/935
+hook after_error_render => sub { setting('layout' => 'main') };
+
+# this hook should be loaded _after_ all plugins
 hook 'before_template' => sub {
     my $tokens = shift;
 
@@ -51,7 +57,14 @@ hook 'before_template' => sub {
         if request->base->path ne '/';
 
     # allow portable dynamic content
-    $tokens->{uri_for} = sub { uri_for(@_)->path_query() };
+    $tokens->{uri_for} = sub { uri_for(@_)->path_query };
+
+    # access to logged in user's roles
+    $tokens->{user_has_role}  = sub { user_has_role(@_) };
+
+    # fix Plugin Template Variables to be only path+query
+    $tokens->{$_} = $tokens->{$_}->path_query
+      for qw/search_node search_device device_ports/;
 
     # allow very long lists of ports
     $Template::Directive::WHILE_MAX = 10_000;
@@ -60,13 +73,24 @@ hook 'before_template' => sub {
     $Template::Stash::PRIVATE = undef;
 };
 
-get '/' => sub {
-    if (var('user') and var('user')->admin) {
-        if (schema('netdisco')->resultset('Device')->count == 0) {
-            var('nodevices' => true);
-        }
-    }
+# remove empty lines from CSV response
+# this makes writing templates much more straightforward!
+hook 'after' => sub {
+    my $r = shift; # a Dancer::Response
 
+    if ($r->content_type and $r->content_type eq 'text/comma-separated-values') {
+        my @newlines = ();
+        my @lines = split m/\n/, $r->content;
+
+        foreach my $line (@lines) {
+            push @newlines, $line if $line !~ m/^\s*$/;
+        }
+
+        $r->content(join "\n", @newlines);
+    }
+};
+
+get qr{^/(?:login(?:/denied)?)?} => sub {
     template 'index';
 };
 
@@ -76,4 +100,15 @@ any qr{.*} => sub {
     template 'index';
 };
 
+{
+  # https://github.com/PerlDancer/Dancer/issues/967
+  no warnings 'redefine';
+  *Dancer::_redirect = sub {
+      my ($destination, $status) = @_;
+      my $response = Dancer::SharedData->response;
+      $response->status($status || 302);
+      $response->headers('Location' => $destination);
+  };
+}
+
 true;
